import { Component, ContentChild, EventEmitter, Injectable, Input, NgModule, Output } from '@angular/core';
import { CommonModule } from '@angular/common';
var CUSTOM_PATTERNS = {
    'url': {
        type: 'url',
        pattern: /(https?:\/\/|www\.)[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&\/=]*[-a-zA-Z0-9@:%_\+~#?&\/=])*/i,
        element: {
            tag: '<a>',
            attributes: {
                target: '_blank',
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                displayText: function (link, $event) {
                    return link;
                },
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                href: function (link, $event) {
                    return link;
                },
            },
        },
    },
    'phone': {
        type: 'phone',
        pattern: /[\+]?[(]?[0-9]{3}[)]?[-\s\.]?[0-9]{3}[-\s\.]?[0-9]{4,7}/,
        element: {
            tag: '<a>',
            attributes: {
                target: '_self',
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                displayText: function (link, $event) {
                    return link;
                },
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                href: function (link, $event) {
                    return "tel:" + link;
                },
            },
        },
    },
    'email': {
        type: 'email',
        pattern: /\S+@\S+\.\S+/,
        element: {
            tag: '<a>',
            attributes: {
                target: '_blank',
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                displayText: function (link, $event) {
                    return link;
                },
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                href: function (link, $event) {
                    return "mailto:" + link;
                },
            },
        },
    },
};
var HighlightService = /** @class */ (function () {
    function HighlightService() {
    }
    /**
     * @param {?} patterns
     * @return {?}
     */
    HighlightService.prototype.linkPatterns = function (patterns) {
        var /** @type {?} */ checkedPatterns = [];
        var /** @type {?} */ types = {};
        patterns.forEach(function (pat) {
            if (pat.pattern instanceof RegExp) {
                checkedPatterns.push(pat);
                if (CUSTOM_PATTERNS[pat.type]) {
                    types[pat.type] = true;
                }
            }
        });
        Object.keys(CUSTOM_PATTERNS).forEach(function (type) {
            if (!types[type]) {
                checkedPatterns.push(CUSTOM_PATTERNS[type]);
            }
        });
        this.PATTERNS = checkedPatterns.slice();
    };
    /**
     * @param {?} text
     * @return {?}
     */
    HighlightService.prototype.parse = function (text) {
        var _this = this;
        var /** @type {?} */ hyperText = [{ text: text, match: false }], /** @type {?} */ str = '';
        this.PATTERNS.forEach(function (patterns) {
            var /** @type {?} */ parts = [];
            hyperText.forEach(function (text) {
                if (text.match) {
                    parts.push(text);
                    return;
                }
                str = text.text;
                while (patterns.pattern.test(str)) {
                    var /** @type {?} */ matches = patterns.pattern.exec(str);
                    var /** @type {?} */ index = matches.index;
                    var /** @type {?} */ left = str.substr(0, index - 1);
                    parts.push({ text: left, match: false });
                    var /** @type {?} */ link = str.substr(index, matches[0].length);
                    parts.push(_this.setParsing(patterns, true, link));
                    str = str.slice(index + matches[0].length + 1);
                }
                if (str) {
                    parts.push({ text: str, match: false });
                }
            });
            hyperText = parts;
        });
        return hyperText;
    };
    /**
     * @param {?} patterns
     * @param {?} match
     * @param {?} link
     * @return {?}
     */
    HighlightService.prototype.setParsing = function (patterns, match, link) {
        var /** @type {?} */ setTag, /** @type {?} */ setAttributes = {};
        var type = patterns.type, pattern = patterns.pattern, _a = patterns.element, tag = _a.tag, attributes = _a.attributes;
        setTag = !patterns.element && (!tag || tag !== '<a>') ? '<span>' : tag;
        if (typeof attributes === 'object') {
            ['target', 'href', 'displayText'].forEach(function (attr) {
                if (attributes[attr] !== undefined) {
                    if (typeof attributes[attr] === 'function') {
                        setAttributes[attr] = attributes[attr](link, { type: type, pattern: pattern });
                    }
                    else {
                        setAttributes[attr] = attributes[attr];
                    }
                }
            });
        }
        if (Object.keys(setAttributes).length === 0) {
            setTag = '<span>';
            setAttributes = { displayText: link };
        }
        return Object.assign({ type: type,
            pattern: pattern,
            match: match,
            link: link, tag: setTag }, setAttributes);
    };
    return HighlightService;
}());
HighlightService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
HighlightService.ctorParameters = function () { return []; };
var ParsedTextComponent = /** @class */ (function () {
    /**
     * @param {?} highlight
     */
    function ParsedTextComponent(highlight) {
        this.highlight = highlight;
        this.trigger = new EventEmitter();
    }
    Object.defineProperty(ParsedTextComponent.prototype, "text", {
        /**
         * @param {?} text
         * @return {?}
         */
        set: function (text) {
            this._text = text;
            this.highlightText();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ParsedTextComponent.prototype, "parse", {
        /**
         * @param {?} patterns
         * @return {?}
         */
        set: function (patterns) {
            this._parse = patterns;
            this.highlightText();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ParsedTextComponent.prototype.ngAfterViewInit = function () {
        this.highlightText();
    };
    /**
     * @return {?}
     */
    ParsedTextComponent.prototype.highlightText = function () {
        if (!this._text || typeof this._text !== 'string') {
            console.error('text is empty or not in correct format');
            return;
        }
        var /** @type {?} */ patterns = [];
        if (Array.isArray(this._parse)) {
            patterns = this._parse;
        }
        this.highlight.linkPatterns(patterns);
        this.parsedList = this.highlight.parse(this._text);
    };
    /**
     * @param {?} parse
     * @return {?}
     */
    ParsedTextComponent.prototype.emitLink = function (parse) {
        this.trigger.emit(parse);
    };
    return ParsedTextComponent;
}());
ParsedTextComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-parsed-text',
                template: "\n  <ng-template #link let-parse>\n    <a\n      class=\"link\"\n      *ngIf=\"parse.tag === '<a>'\"\n      [href]=\"parse.href\"\n      [target]=\"parse.target\">\n        {{parse.displayText}}\n    </a>\n    <a\n      class=\"link clickable\"\n      *ngIf=\"parse.tag !== '<a>'\"\n      (click)=\"emitLink(parse)\"\n    >\n        {{parse.link}}\n    </a>\n  </ng-template>\n  <ng-template ngFor let-item [ngForOf]=\"parsedList\"]>\n    <ng-container *ngIf=\"item.match  else renderText\">\n      <ng-container \n        [ngTemplateOutlet]=\"custom ? custom : link\"\n        [ngTemplateOutletContext]=\"{$implicit: item}\"\n      >\n      </ng-container>\n    </ng-container>\n    <ng-template #renderText>\n      {{ item.text }}\n    </ng-template>\n  </ng-template>",
                styles: [
                    ".link {\n    cursor: pointer;\n    color: #2196f3;\n    text-decoration: none;\n  }\n  :host-context(.ng-parse-text-red) .link {\n      color: #de1919;\n  }\n  :host-context(.ng-parse-text-green) .link {\n      color: #7ebb38\n  }"
                ],
            },] },
];
/**
 * @nocollapse
 */
ParsedTextComponent.ctorParameters = function () { return [
    { type: HighlightService, },
]; };
ParsedTextComponent.propDecorators = {
    'text': [{ type: Input },],
    'parse': [{ type: Input },],
    'trigger': [{ type: Output },],
    'custom': [{ type: ContentChild, args: ['custom',] },],
};
var ParsedTextModule = /** @class */ (function () {
    function ParsedTextModule() {
    }
    return ParsedTextModule;
}());
ParsedTextModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [ParsedTextComponent],
                exports: [ParsedTextComponent],
                providers: [HighlightService]
            },] },
];
/**
 * @nocollapse
 */
ParsedTextModule.ctorParameters = function () { return []; };
/**
 * Generated bundle index. Do not edit.
 */
export { ParsedTextModule, ParsedTextComponent, HighlightService };
//# sourceMappingURL=ng-parsed-text.es5.js.map
